# -*- coding: utf-8 -*-
#
# This file is part of kwalitee
# Copyright (C) 2014, 2015 CERN.
#
# kwalitee is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# kwalitee is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kwalitee; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Integration tests for the pull_request event."""

from __future__ import absolute_import, unicode_literals

import httpretty

from flask import json
from kwalitee.models import CommitStatus, Repository
from kwalitee.tasks import push
from hamcrest import (assert_that, equal_to, contains_string, has_length,
                      has_item)

from utils import MyQueue


def test_push(app, repository):
    """POST /payload (push) performs the checks"""
    queue = MyQueue()
    # Replace the default Redis queue
    app.config["queue"] = queue

    push_event = {
        "commits": [{
            "id": "1",
            "url": "https://github.com/commits/1"
        }, {
            "id": "2",
            "url": "https://github.com/commits/2"
        }],
        "repository": {
            "name": "test",
            "owner": {
                "name": "invenio"
            }
        }
    }

    tester = app.test_client()
    response = tester.post("/payload", content_type="application/json",
                           headers=(("X-GitHub-Event", "push"),
                                    ("X-GitHub-Delivery", "1")),
                           data=json.dumps(push_event))

    assert_that(response.status_code, equal_to(200))
    body = json.loads(response.data)
    assert_that(body["payload"]["state"], equal_to("pending"))

    cs = CommitStatus.query.filter_by(repository_id=repository.id,
                                      sha="1").first()

    (fn, commit_id, commit_url, status_url, config) = queue.dequeue()
    assert_that(fn, equal_to(push))
    assert_that(commit_id, equal_to(cs.id))
    assert_that(commit_url, equal_to("https://api.github.com"
                                     "/repos/invenio/test/commits/1"))
    assert_that(status_url, contains_string("/invenio/test/commits/1"))

    cs = CommitStatus.query.filter_by(repository_id=repository.id,
                                      sha="2").first()

    (fn, commit_id, commit_url, status_url, config) = queue.dequeue()
    assert_that(fn, equal_to(push))
    assert_that(commit_id, equal_to(cs.id))
    assert_that(commit_url, equal_to("https://api.github.com"
                                     "/repos/invenio/test/commits/2"))
    assert_that(status_url, contains_string("/invenio/test/commits/2"))


def test_push_with_auto_create(app, repository):
    """POST /payload (push) performs the checks"""
    queue = MyQueue()
    # Replace the default Redis queue
    config = dict(app.config)
    app.config["queue"] = queue
    app.config["AUTO_CREATE"] = True

    push_event = {
        "commits": [{
            "id": "1",
            "url": "https://github.com/commits/1"
        }],
        "repository": {
            "name": "doe",
            "owner": {
                "name": "john"
            }
        }
    }

    tester = app.test_client()
    response = tester.post("/payload", content_type="application/json",
                           headers=(("X-GitHub-Event", "push"),
                                    ("X-GitHub-Delivery", "1")),
                           data=json.dumps(push_event))

    assert_that(response.status_code, equal_to(200))
    body = json.loads(response.data)
    assert_that(body["payload"]["state"], equal_to("pending"))

    repo = Repository.query.filter_by(name="doe").first()
    assert_that(repo)
    assert_that(repo.owner.name, equal_to("john"))
    app.config = config


def test_push_to_unknown_repository(app):
    """POST /payload (push) with unknown repository should fails."""
    queue = MyQueue()
    config = dict(app.config)
    app.config["queue"] = queue

    push_event = {
        "commits": [{
            "id": "1",
            "url": "https://github.com/commits/1"
        }],
        "repository": {
            "name": "eggs",
            "owner": {
                "name": "spam"
            }
        }
    }

    tester = app.test_client()
    response = tester.post("/payload", content_type="application/json",
                           headers=(("X-GitHub-Event", "push"),
                                    ("X-GitHub-Delivery", "1")),
                           data=json.dumps(push_event))

    assert_that(response.status_code, equal_to(200))
    body = json.loads(response.data)
    assert_that(body["payload"]["state"], equal_to("error"))
    assert_that(body["payload"]["description"],
                contains_string("spam/eggs"))
    assert_that(body["payload"]["context"],
                equal_to(app.config.get("CONTEXT")))

    app.config = config


def test_push_valid_commit(app, repository):
    """Worker push /commits/1 is valid"""
    httpretty.reset()
    commit = {
        "sha": 1,
        "url": "https://api.github.com/commits/1",
        "html_url": "https://github.com/commits/1",
        "comments_url": "https://api.github.com/commits/1/comments",
        "commit": {
            "message": "comp: that\n\nBy: John Doe <john.doe@example.org>",
        },
        "files": [{
            "filename": "spam/__init__.py",
            "status": "added",
            "raw_url": "https://github.com/raw/1/spam/__init__.py"
        }, {
            "filename": "spam/deleted_files.py",
            "status": "removed",
            "raw_url": "https://github.com/raw/1/spam/deleted_files.py"
        }]
    }
    httpretty.register_uri(httpretty.GET,
                           "https://api.github.com/commits/1",
                           body=json.dumps(commit),
                           content_type="application/json")
    init_py = '"""Test module."""\n'
    httpretty.register_uri(httpretty.GET,
                           "https://github.com/raw/1/spam/__init__.py",
                           body=init_py,
                           content_type="text/plain")
    status = {"id": 1, "state": "success"}
    httpretty.register_uri(httpretty.POST,
                           "https://api.github.com/statuses/1",
                           status=201,
                           body=json.dumps(status),
                           content_type="application/json")

    cs = CommitStatus.find_or_create(repository,
                                     commit["sha"],
                                     commit["url"])
    assert_that(cs.is_pending())

    httpretty.enable()
    push(cs.id,
         "https://api.github.com/commits/1",
         "https://api.github.com/statuses/1",
         {"COMPONENTS": ["comp"],
          "SIGNATURES": ["By"],
          "TRUSTED_DEVELOPERS": ["john.doe@example.org"],
          "CHECK_LICENSE": False,
          "repository": repository.id})
    httpretty.disable()

    latest_requests = httpretty.HTTPretty.latest_requests
    assert_that(len(latest_requests), equal_to(3), "2x GET, 1x POST")

    expected_requests = [
        "",
        "",
        "success"
    ]
    for expected, request in zip(expected_requests, latest_requests):
        assert_that(str(request.parsed_body), contains_string(expected))

    cs = CommitStatus.query.filter_by(repository_id=repository.id,
                                      sha=commit["sha"]).first()
    assert_that(cs)
    assert_that(cs.state, equal_to("success"))
    assert_that(cs.errors, equal_to(0))
    assert_that(cs.content["files"]["spam/__init__.py"]["errors"],
                has_length(0))


def test_push_wip_commit(app, repository):
    """Worker push /commits/1 has wip as a component and is ignored"""
    httpretty.reset()
    commit = {
        "sha": 1,
        "url": "https://api.github.com/commits/1",
        "html_url": "https://github.com/commits/1",
        "comments_url": "https://api.github.com/commits/1/comments",
        "commit": {
            "message": "wip: herp derp\n\nBy: John Doe <john.doe@example.org>",
        },
        "files": [{
            "filename": "spam/__init__.py",
            "status": "added",
            "raw_url": "https://github.com/raw/1/spam/__init__.py"
        }]
    }
    httpretty.register_uri(httpretty.GET,
                           "https://api.github.com/commits/1",
                           body=json.dumps(commit),
                           content_type="application/json")
    status = {"id": 1, "state": "success"}
    httpretty.register_uri(httpretty.POST,
                           "https://api.github.com/statuses/1",
                           status=201,
                           body=json.dumps(status),
                           content_type="application/json")

    cs = CommitStatus.find_or_create(repository,
                                     commit["sha"],
                                     commit["url"])
    assert_that(cs.is_pending())

    httpretty.enable()
    push(cs.id,
         "https://api.github.com/commits/1",
         "https://api.github.com/statuses/1",
         {"COMPONENTS": ["comp"],
          "SIGNATURES": ["By"],
          "TRUSTED_DEVELOPERS": ["john.doe@example.org"],
          "CHECK_LICENSE": False,
          "repository": repository.id})
    httpretty.disable()

    latest_requests = httpretty.HTTPretty.latest_requests
    assert_that(len(latest_requests), equal_to(2), "1x GET, 1x POST")

    expected_requests = [
        "",
        "success"
    ]
    for expected, request in zip(expected_requests, latest_requests):
        assert_that(str(request.parsed_body), contains_string(expected))

    cs = CommitStatus.query.filter_by(repository_id=repository.id,
                                      sha=commit["sha"]).first()
    assert_that(cs)
    assert_that(cs.state, equal_to("success"))
    assert_that(cs.errors, equal_to(0))


def test_push_broken_commit_message(app, repository):
    """Worker push /commits/1 is invalid (message)"""
    httpretty.reset()
    commit = {
        "sha": 1,
        "url": "https://api.github.com/commits/1",
        "html_url": "https://github.com/commits/1",
        "comments_url": "https://api.github.com/commits/1/comments",
        "commit": {
            "message": "Fix all the bugs!"
        },
        "files": [{
            "filename": "spam/eggs.py",
            "status": "modified",
            "raw_url": "https://github.com/raw/1/spam/eggs.py"
        }]
    }
    httpretty.register_uri(httpretty.GET,
                           "https://api.github.com/commits/1",
                           body=json.dumps(commit),
                           content_type="application/json")
    eggs_py = '"""Eggs are boiled."""\n'
    httpretty.register_uri(httpretty.GET,
                           "https://github.com/raw/1/spam/eggs.py",
                           body=eggs_py,
                           content_type="text/plain")
    httpretty.register_uri(httpretty.POST,
                           "https://api.github.com/commits/1/comments",
                           status=201,
                           body=json.dumps({"id": 1}),
                           content_type="application/json")
    status = {"id": 1, "state": "success"}
    httpretty.register_uri(httpretty.POST,
                           "https://api.github.com/statuses/1",
                           status=201,
                           body=json.dumps(status),
                           content_type="application/json")

    cs = CommitStatus.find_or_create(repository,
                                     commit["sha"],
                                     commit["url"])
    assert_that(cs.is_pending())

    httpretty.enable()
    push(cs.id,
         "https://api.github.com/commits/1",
         "https://api.github.com/statuses/1",
         {"CHECK_LICENSE": False,
          "repository": repository.id})
    httpretty.disable()

    latest_requests = httpretty.HTTPretty.latest_requests
    assert_that(len(latest_requests), equal_to(4), "2x GET, 2x POST")

    expected_requests = [
        "",
        "needs more reviewers",
        "",
        "error"
    ]
    for expected, request in zip(expected_requests, latest_requests):
        assert_that(str(request.parsed_body),
                    contains_string(expected))


def test_push_broken_files(repository):
    """Worker push /commits/1 is invalid (files)"""
    httpretty.reset()
    commit = {
        "sha": 1,
        "url": "https://api.github.com/commits/1",
        "html_url": "https://github.com/commits/1",
        "comments_url": "https://api.github.com/commits/1/comments",
        "commit": {
            "message": "comp: bob\n\nBy: John <john.doe@example.org>"
        },
        "files": [{
            "filename": "spam/eggs.py",
            "status": "modified",
            "raw_url": "https://github.com/raw/1/spam/eggs.py"
        }]
    }
    httpretty.register_uri(httpretty.GET,
                           "https://api.github.com/commits/1",
                           body=json.dumps(commit),
                           content_type="application/json")
    eggs_py = "if foo == bar:\n  print('derp')\n"
    httpretty.register_uri(httpretty.GET,
                           "https://github.com/raw/1/spam/eggs.py",
                           body=eggs_py,
                           content_type="text/plain")
    httpretty.register_uri(httpretty.POST,
                           "https://api.github.com/commits/1/comments",
                           status=201,
                           body=json.dumps({"id": 1}),
                           content_type="application/json")
    status = {"id": 1, "state": "success"}
    httpretty.register_uri(httpretty.POST,
                           "https://api.github.com/statuses/1",
                           status=201,
                           body=json.dumps(status),
                           content_type="application/json")

    cs = CommitStatus.find_or_create(repository,
                                     commit["sha"],
                                     commit["url"])
    assert_that(cs.is_pending())

    httpretty.enable()
    push(cs.id,
         "https://api.github.com/commits/1",
         "https://api.github.com/statuses/1",
         {"COMPONENTS": ["comp"],
          "SIGNATURES": ["By"],
          "TRUSTED_DEVELOPERS": ["john.doe@example.org"],
          "repository": repository.id})
    httpretty.disable()

    latest_requests = httpretty.HTTPretty.latest_requests
    assert_that(len(latest_requests), equal_to(4), "2x GET, 2x POST")

    expected_requests = [
        "",
        "",
        "F821 undefined name 'foo'",
        "error"
    ]
    for expected, request in zip(expected_requests, latest_requests):
        assert_that(str(request.parsed_body),
                    contains_string(expected))


def test_push_known_commit(repository, session):
    """Worker push /commits/1 is not rechecked if known"""
    httpretty.reset()
    commit = {
        "sha": 1,
        "url": "https://api.github.com/commits/1",
        "html_url": "https://github.com/commits/1",
        "comments_url": "https://api.github.com/commits/1/comments",
        "commit": {
            "message": "Fix all the bugs!"
        },
        "files": [{
            "filename": "spam/eggs.py",
            "status": "modified",
            "raw_url": "https://github.com/raw/1/spam/eggs.py"
        }]
    }
    httpretty.register_uri(httpretty.GET,
                           "https://api.github.com/commits/1",
                           body=json.dumps(commit),
                           content_type="application/json")

    cs = CommitStatus(repository,
                      "1",
                      "https://github.com/commits/1",
                      {"message": ["error 1", "error 2"], "files": {}})
    session.add(cs)
    session.commit()
    assert_that(cs.is_pending(), equal_to(False))

    httpretty.enable()
    body = push(cs.id,
                "https://api.github.com/commits/1",
                "https://api.github.com/statuses/1",
                {"repository": repository.id})
    httpretty.disable()

    latest_requests = httpretty.HTTPretty.latest_requests
    assert_that(len(latest_requests), equal_to(1), "1x GET")

    assert_that(body["description"],
                contains_string("[error] 2 errors"))


def test_push_half_known_commit(repository, session):
    """Worker push /commits/1 checks the files if none"""
    httpretty.reset()
    commit = {
        "sha": "1",
        "url": "https://api.github.com/commits/1",
        "html_url": "https://github.com/commits/1",
        "comments_url": "https://api.github.com/commits/1/comments",
        "commit": {
            "message": "Fix all the bugs!"
        },
        "files": [{
            "filename": "spam/eggs.py",
            "status": "modified",
            "raw_url": "https://github.com/raw/1/spam/eggs.py"
        }]
    }
    httpretty.register_uri(httpretty.GET,
                           "https://api.github.com/commits/1",
                           body=json.dumps(commit),
                           content_type="application/json")
    eggs_py = "if foo == bar:\n  print('derp')\n"
    httpretty.register_uri(httpretty.GET,
                           "https://github.com/raw/1/spam/eggs.py",
                           body=eggs_py,
                           content_type="text/plain")
    httpretty.register_uri(httpretty.POST,
                           "https://api.github.com/commits/1/comments",
                           status=201,
                           body=json.dumps({"id": 1}),
                           content_type="application/json")
    status = {"id": 1, "state": "success"}
    httpretty.register_uri(httpretty.POST,
                           "https://api.github.com/statuses/1",
                           status=201,
                           body=json.dumps(status),
                           content_type="application/json")

    cs = CommitStatus(repository,
                      "1",
                      "https://github.com/commits/1",
                      {"message": [], "files": None})
    session.add(cs)
    session.commit()
    assert_that(cs.is_pending(), equal_to(False))

    httpretty.enable()
    push(cs.id,
         "https://api.github.com/commits/1",
         "https://api.github.com/statuses/1",
         {"repository": repository.id})
    httpretty.disable()

    latest_requests = httpretty.HTTPretty.latest_requests
    assert_that(len(latest_requests), equal_to(4), "2x GET, 2x POST")

    expected_requests = [
        "",
        "",
        "F821 undefined name 'foo'",
        "error"
    ]
    for expected, request in zip(expected_requests, latest_requests):
        assert_that(str(request.parsed_body),
                    contains_string(expected))

    cs = CommitStatus.query.filter_by(id=cs.id).first()
    assert_that(cs)
    assert_that(cs.is_pending(), equal_to(False))
    assert_that(cs.content["files"]["spam/eggs.py"]["errors"],
                has_item("1: D100 Missing docstring in public module"))



# -*- coding: utf-8 -*-
#
# This file is part of kwalitee
# Copyright (C) 2014, 2015 CERN.
#
# kwalitee is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# kwalitee is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#g
# You should have received a copy of the GNU General Public License
# along with kwalitee; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Command-line tools for managing accounts."""

from __future__ import absolute_import, print_function, unicode_literals

import sys

from ..models import db, Account

from flask_script import Manager

manager = Manager(usage="repository management")


@manager.command
@manager.option('-m', '--email', dest="email", required=False)
@manager.option('-t', '--token', dest="token", required=False)
def add(account, email=None, token=None):
    """Add/modify an account."""
    acc = Account.update_or_create(account, email, token)

    print("Welcome (back) {0.name}!".format(acc), file=sys.stderr)


@manager.command
def remove(account):
    """Remove an account (and its repositories)."""
    acc = Account.query.filter_by(name=account).first()
    if not acc:
        return

    db.session.delete(acc)
    db.session.commit()


@manager.command
def list():
    """List all the repositories."""
    for acc in Account.query.order_by(db.asc(Account.name)).all():
        if acc.email:
            print("{acc.name} <{acc.email}>".format(acc=acc))
        else:
            print(acc.name)



# -*- coding: utf-8 -*-
#
# This file is part of kwalitee
# Copyright (C) 2014 CERN.
#
# kwalitee is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# kwalitee is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kwalitee; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

from __future__ import unicode_literals

from hamcrest import (assert_that, equal_to, contains_string,
                      string_contains_in_order)
from kwalitee.models import Account, Repository
from kwalitee.cli.repository import add, remove, list as list_


def test_add(capsys, session):
    add("invenio/kwalitee")
    _, err = capsys.readouterr()
    assert_that(err, contains_string("invenio/kwalitee"))
    assert_that(Account.query.filter_by(name="invenio").count(),
                equal_to(1))
    assert_that(Repository.query.filter_by(name="kwalitee").count(),
                equal_to(1))


def test_add_existing(session):
    add("invenio/kwalitee")
    add("invenio/kwalitee")
    assert_that(Account.query.filter_by(name="invenio").count(),
                equal_to(1))
    assert_that(Repository.query.filter_by(name="kwalitee").count(),
                equal_to(1))


def test_add_invalid_name(capsys, session):
    assert_that(add("foo"), equal_to(1))
    _, err = capsys.readouterr()
    assert_that(err, contains_string("foo is not a valid repository"))


def test_remove(session):
    add("invenio/kwalitee")
    remove("invenio/kwalitee")
    remove("invenio/kwalitee")
    remove("kwalitee/kwalitee")
    assert_that(Account.query.filter_by(name="invenio").count(),
                equal_to(1))
    assert_that(Repository.query.filter_by(name="kwalitee").count(),
                equal_to(0))


def test_remove_invalid_name(capsys, session):
    assert_that(remove("foo"), equal_to(1))
    _, err = capsys.readouterr()
    assert_that(err, contains_string("foo is not a valid repository"))


def test_list(capsys, session):
    repos = ["a/a", "bcd/def", "foo-bar/spam-and-eggs", "foo-bar/zoo"]
    for i in range(len(repos)):
        add(repos[-i])
    list_()
    out, _ = capsys.readouterr()
    assert_that(out, string_contains_in_order(*repos))



# -*- coding: utf-8 -*-
#
# This file is part of kwalitee
# Copyright (C) 2015 CERN.
#
# kwalitee is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# kwalitee is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kwalitee; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Test excludes option."""

from __future__ import unicode_literals

import os

from kwalitee.kwalitee import check_file, is_file_excluded, get_options

from unittest import TestCase


class TestExcludesOption(TestCase):

    """Test excludes option."""

    def setUp(self):
        """Setup."""
        base_dir = os.path.join(os.path.dirname(__file__), "fixtures",
                                "excludes_option")
        self.legacy_regex_1 = ['(.)+/legacy/(.)+']
        self.legacy_regex_2 = ['(.)+/legacy/(.)+', '(.)+/foo/(.)+']
        self.file_1 = base_dir + '/fuu/bar/foo.py'
        self.file_2 = base_dir + '/foo/bar/beer.py'
        self.file_3 = base_dir + '/legacy/excluded/excluded.py'

    def test_options(self):
        """Check option loading."""
        # test default value
        options = get_options({})
        assert options['excludes'] == []
        # test set excludes
        options = get_options({'EXCLUDES': self.legacy_regex_1})
        assert options['excludes'] == self.legacy_regex_1

    def test_is_file_escluded(self):
        """Check if is_file_escluded() correctly excludes files."""
        assert is_file_excluded(filename=self.file_1,
                                excludes=self.legacy_regex_1) is False
        assert is_file_excluded(filename=self.file_1,
                                excludes=self.legacy_regex_2) is False
        assert is_file_excluded(filename=self.file_2,
                                excludes=self.legacy_regex_1) is False
        assert is_file_excluded(filename=self.file_2,
                                excludes=self.legacy_regex_2) is True
        assert is_file_excluded(filename=self.file_3,
                                excludes=self.legacy_regex_1) is True
        assert is_file_excluded(filename=self.file_3,
                                excludes=self.legacy_regex_2) is True

    def test_check_file(self):
        """Test check_file()."""
        options_1 = get_options({'EXCLUDES': self.legacy_regex_1})
        options_2 = get_options({'EXCLUDES': self.legacy_regex_2})

        assert check_file(filename=self.file_1, **options_1) is not None
        assert check_file(filename=self.file_1, **options_2) is not None
        assert check_file(filename=self.file_2, **options_1) is not None
        assert check_file(filename=self.file_2, **options_2) is None
        assert check_file(filename=self.file_3, **options_1) is None
        assert check_file(filename=self.file_3, **options_2) is None



# -*- coding: utf-8 -*-
#
# This file is part of kwalitee
# Copyright (C) 2014 CERN.
#
# kwalitee is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# kwalitee is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kwalitee; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Integration tests for the commit page."""

from __future__ import unicode_literals

import pytest
from kwalitee.models import CommitStatus
from hamcrest import assert_that, equal_to, contains_string


@pytest.fixture(scope="function")
def commit(repository, session, request):
    sha = "060fcc4e4856b1bc7dc0eba0b07d3c734bbf24fd"
    url_template = "https://github.com/invenio/test/commits/{sha}"

    c = CommitStatus(repository,
                     sha,
                     url_template.format(sha=sha),
                     {"message": ["foo", "bar"],
                      "files": {"spam": {"errors": ["spam"]},
                                "eggs": {"errors": ["eggs"]}}})
    session.add(c)
    session.commit()

    def teardown():
        session.delete(c)
        session.commit()

    request.addfinalizer(teardown)
    return c


def test_get_commit(app, owner, repository, commit):
    """GET /{account}/{repository}/commits/{sha} displays the commit."""

    tester = app.test_client()
    response = tester.get("/{0}/{1}/commits/{2}/".format(
                          owner.name,
                          repository.name,
                          commit.sha))

    assert_that(response.status_code, equal_to(200))
    body = response.get_data(as_text=True)
    assert_that(body,
                contains_string("/{0}/{1}/".format(
                                owner.name,
                                repository.name)))
    assert_that(body,
                contains_string("/commits/{sha}".format(sha=commit.sha)))
    assert_that(body, contains_string("foo"))
    assert_that(body, contains_string("eggs"))


def test_get_commit_doesnt_exist(app):
    """GET /{account}/{repository}/commits/404 raise 404 if not found."""

    tester = app.test_client()
    response = tester.get("/invenio/test/commits/404/")

    assert_that(response.status_code, equal_to(404))



# -*- coding: utf-8 -*-
#
# This file is part of kwalitee
# Copyright (C) 2014, 2015 CERN.
#
# kwalitee is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# kwalitee is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kwalitee; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

from __future__ import unicode_literals

from itertools import repeat
from unittest import TestCase

from hamcrest import assert_that, has_item, has_items, has_length, is_not

from kwalitee.kwalitee import check_message


class TestCheckMessage(TestCase):
    """Unit tests of the commit message validation checks"""
    options = dict(components=("search", "utils"),
                   trusted=("john.doe@example.org",),
                   signatures=("Signed-off-by", "Reviewed-by"),
                   commit_msg_labels=(('NEW', 'New'), ('AMENDS', None)),
                   alt_signatures=("Reported-by",))

    def test_no_message(self):
        errors = check_message("", **self.options)
        assert_that(errors,
                    has_items("1: M110 missing component name",
                              "1: M101 signature is missing",
                              "1: M100 needs more reviewers"))

    def test_allow_empty_message(self):
        options = dict(self.options, allow_empty="True")
        errors = check_message("", **options)
        assert_that(errors, has_length(0))

    def test_no_component_name(self):
        errors = check_message("foo bar.", **self.options)
        assert_that(errors,
                    has_items("1: M110 missing component name",
                              "1: M101 signature is missing",
                              "1: M100 needs more reviewers",
                              "1: M191 must not end with a dot '.'"))

    def test_known_component_name(self):
        errors = check_message("utils: foo bar", **self.options)
        assert_that(errors,
                    has_items("1: M101 signature is missing",
                              "1: M100 needs more reviewers"))

    def test_unknonwn_component_name(self):
        errors = check_message("kikoo: lol", **self.options)
        assert_that(errors,
                    has_item("1: M111 unrecognized component name: kikoo"))

    def test_first_line_is_too_long(self):
        message = "".join(repeat("M", 51))
        errors = check_message(message, **self.options)
        assert_that(errors, has_item("1: M190 line is too long (51 > 50)"))

    def test_has_enough_reviewers(self):
        errors = check_message("search: hello\r\n\r\n"
                               "Signed-off-by: a a <a@a.com>\r\n"
                               "Reviewed-by: b b <b@b.com>\r\n"
                               "Reviewed-by: c c <c@c.com>\r\n"
                               "Reported-by: d d <d@d.com>",
                               **self.options)
        assert_that(errors, has_length(0))

    def test_needs_more_reviewers(self):
        errors = check_message("search: hello\r\n\r\n"
                               "Signed-off-by: a a <a@a.com>",
                               **self.options)
        assert_that(errors, has_item("1: M100 needs more reviewers"))

    def test_signature_ends_with_a_dot(self):
        errors = check_message("search: hello\r\n\r\n"
                               "Signed-off-by: a a <john.doe@example.org>.",
                               **self.options)
        assert_that(errors, has_item("3: M191 must not end with a dot '.'"))

    def test_has_a_trusted_developer(self):
        errors = check_message("search: hello\r\n\r\n"
                               "Signed-off-by: a a <john.doe@example.org>",
                               **self.options)
        assert_that(errors, has_length(0))

    def test_valid_bullets(self):
        errors = check_message("search: hello\r\n\r\n"
                               "* bullet 1\r\n\r\n"
                               "* bullet 2\r\n\r\n"
                               "* bullet 3\r\n\r\n"
                               "Signed-off-by: a a <john.doe@example.org>",
                               **self.options)
        assert_that(errors, has_length(0))

    def test_valid_multiline_bullets(self):
        errors = check_message("search: hello\r\n\r\n"
                               "* bullet 1\r\n"
                               "  lorem ipsum\r\n\r\n"
                               "* bullet 2\r\n"
                               "  dolor sit amet\r\n\r\n"
                               "Signed-off-by: a a <john.doe@example.org>",
                               **self.options)
        assert_that(errors, has_length(0))

    def test_line_is_too_long(self):
        # max is 72 total including the identation
        too_long = "".join(list(repeat("M", 70)))
        errors = check_message("search: hello\r\n\r\n"
                               "* {0}\r\n\r\n"
                               "* M{0}\r\n\r\n"
                               "Signed-off-by: a a <john.doe@example.org>"
                               .format(too_long),
                               **self.options)
        assert_that(errors, has_length(1))
        assert_that(errors,
                    has_item("5: M190 line is too long (73 > 72)"))

    def test_missing_empty_line_before_bullet(self):
        errors = check_message("search: hello\r\n"
                               "* bullet 1\r\n\r\n"
                               "* bullet 2\r\n"
                               "* bullet 3\r\n\r\n"
                               "Signed-off-by: a a <john.doe@example.org>",
                               **self.options)
        assert_that(errors, has_length(3))
        assert_that(errors,
                    has_items("2: M120 missing empty line before bullet",
                              "5: M120 missing empty line before bullet"))

    def test_using_alternative_bullet_character(self):
        errors = check_message("search: hello\r\n\r\n"
                               "- bullet 1\r\n\r\n"
                               "Signed-off-by: a a <john.doe@example.org>",
                               **self.options)
        assert_that(errors,
                    has_item("3: M102 unrecognized bullet/signature"))

    def test_wrong_identation_after_bullet(self):
        errors = check_message("search: hello\r\n\r\n"
                               "* line 1\r\n"
                               "line 2\r\n"
                               " line 3\r\n"
                               "  line 4\r\n"
                               "   line 5\r\n\r\n"
                               "Signed-off-by: a a <john.doe@example.org>",
                               **self.options)
        assert_that(errors,
                    has_items("4: M121 indentation of two spaces expected",
                              "5: M121 indentation of two spaces expected",
                              "7: M121 indentation of two spaces expected"))
        assert_that(errors,
                    is_not(has_item("M121: 6: indentation of two spaces "
                                    "expected")))

    def test_signatures_mixed_with_bullets(self):
        errors = check_message("search: hello\r\n\r\n"
                               "* bullet 1\r\n\r\n"
                               "Signed-off-by: a a <a@a.com>\r\n\r\n"
                               "* bullet 2\r\n\r\n"
                               "Reviewed-by: b b <b@b.com>\r\n\r\n"
                               "* bullet 3\r\n\r\n"
                               "Reviewed-by: c c <c@c.com>",
                               **self.options)
        assert_that(errors, has_items(
            "7: M130 no bullets are allowed after signatures",
            "11: M130 no bullets are allowed after signatures"))

    def test_bullet_labels(self):
        errors = check_message("search: hello\r\n\r\n"
                               "* NEW bullet 1\r\n  line 2\r\n\r\n"
                               "* AMENDS deadbeef\r\n\r\n"
                               "* DEADBEEF invalid name\r\n\r\n"
                               "* MISSINGSPACE\r\n\r\n"
                               "Signed-off-by: a a <a@a.com>\r\n\r\n"
                               "Reviewed-by: b b <b@b.com>\r\n\r\n"
                               "Reviewed-by: c c <c@c.com>",
                               **self.options)
        assert_that(errors, has_items(
            "8: M122 unrecognized bullet label: DEADBEEF"))



# -*- coding: utf-8 -*-
#
# This file is part of kwalitee
# Copyright (C) 2014, 2015 CERN.
#
# kwalitee is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# kwalitee is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kwalitee; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Kwalitee is a tool that runs static analysis checks on Git repository."""

import os
import sys

from setuptools import setup
from setuptools.command.test import test as TestCommand


install_requires = [
    'alembic',
    'colorama',
    'Flask',
    'Flask-Script',
    'Flask-SQLAlchemy',
    'pep8',
    'pep8-naming',
    'pep257>=0.5.0',
    'pyflakes',
    'flake8-import-order',
    'flake8-blind-except',
    'pytest',
    'requests',
    'rq>=0.4.6',
    'PyYAML',
]

if tuple(sys.version_info) < (3, 0):
    # If pygit2 is not installed, grab GitPython instead.
    try:
        import pygit2
    except ImportError:
        install_requires.append('GitPython>=0.3.2.RC1')

test_requires = [
    'pytest-cov',
    'httpretty',
    'mock',
    'pyhamcrest'
]


# Get the version string.  Cannot be done with import!
version = {}
with open(os.path.join('kwalitee', 'version.py'), 'r') as fp:
    exec(fp.read(), version)


class PyTest(TestCommand):

    """PyTest test runner.

    See: http://pytest.org/latest/goodpractises.html?highlight=setuptools
    """

    user_options = [('pytest-args=', 'a', 'Arguments to pass to py.test')]

    def initialize_options(self):
        """Initialize test options."""
        TestCommand.initialize_options(self)
        self.pytest_args = ["tests"]

    def finalize_options(self):
        """Finalize options."""
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        """Run listed tests."""
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


setup(
    name='kwalitee',
    version=version['__version__'],
    url='https://github.com/inveniosoftware/kwalitee',
    license='GPLv2',
    author='Invenio collaboration',
    author_email='info@invenio-software.org',
    description=__doc__,
    long_description=open('README.rst').read(),
    packages=['kwalitee'],
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=install_requires,
    tests_require=test_requires,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    entry_points={
        'console_scripts': [
            'kwalitee = kwalitee.cli:main',
        ],
    },
    cmdclass={
        'test': PyTest
    },
)



# -*- coding: utf-8 -*-
#
# This file is part of kwalitee
# Copyright (C) 2014 CERN.
#
# kwalitee is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# kwalitee is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kwalitee; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Integration tests for the status page.

Legacy view: that will eventually be dropped.
"""

from __future__ import unicode_literals

import os
import shutil
import tempfile
from hamcrest import assert_that, equal_to, contains_string


def test_simple_status(app):
    """GET /status/sha1 displays the associated text file"""
    sha = "deadbeef"
    instance_path = tempfile.mkdtemp()
    app.instance_path = instance_path
    filename = os.path.join(app.instance_path,
                            "status_{0}.txt".format(sha))
    with open(filename, "w+") as f:
        f.write("\n".join(["{0}: Signature missing",
                           "{0}: Needs more reviewers"]).format(sha))

    tester = app.test_client()
    response = tester.get("status/{0}".format(sha))

    assert_that(response.status_code, equal_to(200))
    assert_that(str(response.data), contains_string("Signature missing"))
    assert_that(str(response.data),
                contains_string("Needs more reviewers"))

    shutil.rmtree(instance_path)


def test_missing_status(app):
    """GET /status/sha2 404"""
    sha = "deadbeef"
    instance_path = tempfile.mkdtemp()
    app.instance_path = instance_path

    tester = app.test_client()
    response = tester.get("status/{0}".format(sha))

    assert_that(response.status_code, equal_to(404))
    shutil.rmtree(instance_path)



# -*- coding: utf-8 -*-
#
# This file is part of kwalitee
# Copyright (C) 2014 CERN.
#
# kwalitee is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# kwalitee is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kwalitee; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

from __future__ import unicode_literals

from hamcrest import (assert_that, equal_to, contains_string,
                      string_contains_in_order)

from kwalitee.models import (Account, Repository, CommitStatus,
                                     BranchStatus)
from kwalitee.cli.account import add, remove, list as list_


def test_add(capsys, session):
    add("invenio", "invenio@example.org", "123")
    _, err = capsys.readouterr()
    assert_that(err, contains_string("invenio"))
    assert_that(Account.query.filter_by(name="invenio",
                                        email="invenio@example.org",
                                        token="123").count(),
                equal_to(1))


def test_add_existing(session):
    add("invenio", "test@example.org", "123")
    add("invenio", "invenio@example.org")
    add("invenio", token="456")
    assert_that(Account.query.filter_by(name="invenio",
                                        email="invenio@example.org",
                                        token="456").count(),
                equal_to(1))


def test_remove(session):
    """remove {account} drops everything tied to it"""
    add("invenio")

    acc = Account.query.filter_by(name="invenio").first()
    rep = Repository.find_or_create(acc, "foo")
    commit = CommitStatus(rep, "1", "http://")
    session.add(commit)
    session.commit()
    bs = BranchStatus(commit, "1", "http://")
    session.add(bs)
    session.commit()

    remove("invenio")

    assert_that(Account.query.filter_by(name="invenio").count(),
                equal_to(0))
    assert_that(Repository.query.filter_by(name="foo").count(),
                equal_to(0))
    assert_that(CommitStatus.query.filter_by(repository_id=rep.id).count(),
                equal_to(0))
    assert_that(BranchStatus.query.filter_by(commit_id=commit.id).count(),
                equal_to(0))


def test_double_remove(session):
    add("invenio")
    remove("invenio")
    remove("invenio")
    assert_that(Account.query.filter_by(name="invenio").count(),
                equal_to(0))


def test_list(capsys, session):
    repos = ["a", "bc", "def", "ghijklmop"]
    for i in range(len(repos)):
        add(repos[-i])
    add("test", "test@тест.укр")
    list_()
    out, _ = capsys.readouterr()
    assert_that(out, string_contains_in_order(*repos))
    assert_that(out, contains_string("test@тест.укр"))



# -*- coding: utf-8 -*-
#
# This file is part of kwalitee
# Copyright (C) 2014, 2015 CERN.
#
# kwalitee is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# kwalitee is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kwalitee; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Views like in MTV."""

from __future__ import unicode_literals

import requests
from flask import (current_app, render_template, make_response, json, jsonify,
                   request, url_for)
from werkzeug.exceptions import NotFound

from .tasks import pull_request, push, get_headers
from .models import db, Account, BranchStatus, CommitStatus, Repository


def status(sha):
    """Show the status of a commit.

    **deprecated** static files aren't used anymore. To be removed at some
    point.

    :param sha: identifier of a commit.
    """
    try:
        with current_app.open_instance_resource(
                "status_{sha}.txt".format(sha=sha), "r") as f:
            status = f.read()
    except IOError:
        raise NotFound("{sha} was not found.".format(sha=sha))

    status = status if len(status) > 0 else sha + ": Everything OK"
    return render_template("status.html", status=status)


def index():
    """Homepage that lists the accounts."""
    accounts = Account.query.order_by(db.asc(Account.name)).all()
    return render_template("index.html", accounts=accounts)


def account(account):
    """Display the repositories linked with one account.

    :param account: name of the account
    """
    acc = _get_account(account)
    return render_template("account.html",
                           account=acc,
                           repositories=acc.repositories)


def repository(account, repository, limit=50):
    """Display the recents commits and branches of a repository.

    :param account: name of the owner
    :param repository: name of the repository
    :param limit: size of the commit window
    """
    acc = _get_account(account)
    repo = _get_repository(acc, repository)
    commits = CommitStatus.query \
                          .filter_by(repository_id=repo.id) \
                          .order_by(db.desc(CommitStatus.id)) \
                          .limit(limit)
    return render_template("repository.html",
                           account=acc,
                           repository=repo,
                           commits=commits)


def commit(account, repository, sha):
    """Display the status of a commit.

    :param account: name of the owner
    :param repository: name of the repository
    :param sha: identifier of the commit
    """
    acc = _get_account(account)
    repo = _get_repository(acc, repository)
    commit = CommitStatus.query.filter_by(repository_id=repo.id,
                                          sha=sha).first_or_404()
    return render_template("commit.html",
                           account=acc,
                           repository=repo,
                           commit=commit)


def branch(account, repository, branch):
    """Display the statuses of a branch.

    :param account: name of the owner
    :param repository: name of the repository
    :param branch: name of the branch
    """
    acc = _get_account(account)
    repo = _get_repository(acc, repository)
    all = BranchStatus.query.join(BranchStatus.commit) \
                            .filter(CommitStatus.repository_id == repo.id) \
                            .filter(BranchStatus.name == branch) \
                            .all()

    if not all:
        raise NotFound("{0.fullname} as no branches called {1}"
                       .format(repo, branch))

    return render_template("branches.html",
                           account=acc,
                           repository=repo,
                           branches=all)


def branch_status(account, repository, branch, sha):
    """Display the status of a pull request.

    :param account: name of the owner
    :param repository: name of the repository
    :param branch: name of the branch
    :param sha: commit identifier of the commit related with the branch
    """
    acc = _get_account(account)
    repo = _get_repository(acc, repository)
    branch = BranchStatus.query.join(BranchStatus.commit) \
                               .filter(CommitStatus.repository_id == repo.id) \
                               .filter(CommitStatus.sha == sha) \
                               .filter(BranchStatus.name == branch) \
                               .first_or_404()

    return render_template("branch.html",
                           account=acc,
                           repository=repo,
                           branch=branch,
                           commit=branch.commit)


def payload():
    """Handle the GitHub events.

    .. seealso::

        `Event Types <https://developer.github.com/v3/activity/events/types/>`
    """
    q = current_app.config["queue"]
    events = ["push", "pull_request"]
    try:
        event = None
        if "X-GitHub-Event" in request.headers:
            event = request.headers["X-GitHub-Event"]
        else:
            raise ValueError("No X-GitHub-Event HTTP header found")

        if event == "ping":
            payload = {"message": "pong"}
        elif event in events:
            config = dict(current_app.config)
            config.pop("queue")
            timeout = config.pop("WORKER_TIMEOUT", None)
            auto_create = config.pop("AUTO_CREATE", False)

            data = json.loads(request.data)

            repository_name = data["repository"]["name"]
            keyname = "name" if event == "push" else "login"
            owner_name = data["repository"]["owner"][keyname]

            payload = {
                "state": "pending",
                "context": config.get("CONTEXT")
            }

            owner = Account.query.filter_by(name=owner_name).first()
            if owner:
                repository = Repository.query.filter_by(
                    name=repository_name,
                    owner_id=owner.id).first()

            if not owner or not repository:
                if auto_create:
                    owner = Account.find_or_create(owner_name)
                    repository = Repository.find_or_create(owner,
                                                           repository_name)
                else:
                    payload["state"] = "error"
                    payload["description"] = "{0}/{1} is not yet registered" \
                                             .format(owner_name,
                                                     repository_name)

            if owner and repository:
                if event == "push":
                    status_url = ""
                    commit_url = "https://api.github.com/repos/{owner}" \
                                 "/{repo}/commits/{sha}"
                    for commit in data["commits"]:
                        cs = CommitStatus.find_or_create(repository,
                                                         commit["id"],
                                                         commit["url"])

                        status_url = url_for("commit",
                                             account=owner.name,
                                             repository=repository.name,
                                             sha=cs.sha,
                                             _external=True)

                        url = commit_url.format(
                            commit_url,
                            owner=owner.name,
                            repo=repository.name,
                            sha=cs.sha)

                        q.enqueue(push, cs.id, url, status_url, config,
                                  timeout=timeout)

                    payload["target_url"] = status_url
                    payload["description"] = "commits queues"

                elif event == "pull_request":
                    if data["action"] not in ["synchronize", "opened",
                                              "reopened"]:
                        raise ValueError(
                            "Pull request action {0} is not supported"
                            .format(data["action"]))
                    repo = data["repository"]
                    data = data["pull_request"]
                    pull_request_url = data["url"]
                    commit_sha = data["head"]["sha"]
                    commits = []
                    headers = get_headers(Repository.query.filter_by(
                        name=repo["name"]).first(), config)
                    response = requests.get(data["commits_url"],
                                            headers=headers)
                    response.raise_for_status()  # check API rate limit
                    response_json = json.loads(response.content)
                    for commit in response_json:
                        cstat = CommitStatus.find_or_create(repository,
                                                            commit["sha"],
                                                            commit["html_url"])
                        commits.append(cstat)

                    bs = BranchStatus.find_or_create(commits[-1],
                                                     data["head"]["label"],
                                                     data["html_url"],
                                                     {"commits": commits})
                    status_url = url_for("branch_status",
                                         account=owner.name,
                                         repository=repository.name,
                                         branch=bs.name,
                                         sha=commit_sha,
                                         _external=True)

                    q.enqueue(pull_request, bs.id, pull_request_url,
                              status_url, config, timeout=timeout)

                    payload["target_url"] = status_url
                    payload["description"] = "pull request {0} queued" \
                                             .format(bs.name)
        else:
            raise ValueError("Event {0} is not supported".format(event))

        return jsonify(payload=payload)
    except Exception as e:
        import traceback
        # Uncomment to help you debugging the tests
        # raise e
        return make_response(jsonify(status="failure",
                                     stacktrace=traceback.format_exc(),
                                     exception=str(e)),
                             500)


def _get_account(account_name):
    """Get the account by name.

    :param account_name: name of the account
    :raise NotFound: if the account cannot be found
    """
    account = Account.query.filter_by(name=account_name).first()
    if not account:
        raise NotFound("{0} isn't registered yet.".format(account_name))
    return account


def _get_repository(account, repository_name):
    """Get the repository by name.

    :param account: account
    :param repository_name: name of the repository
    :raise NotFound: if the repository cannot be found
    """
    repository = Repository.query.filter_by(owner_id=account.id,
                                            name=repository_name).first()
    if not repository:
        raise NotFound("{0}/{1} isn't registered yet.".format(account.name,
                                                              repository_name))
    return repository



# -*- coding: utf-8 -*-
#
# This file is part of kwalitee
# Copyright (C) 2014 CERN.
#
# kwalitee is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# kwalitee is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kwalitee; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

from __future__ import unicode_literals

from hamcrest import assert_that, equal_to, contains_string


def test_get_repository(app, owner, repository, commits, branch):
    """GET /{account}/{repository} displays the recent commits."""

    tester = app.test_client()
    response = tester.get("/{0}/{1}/".format(owner.name,
                                             repository.name))

    assert_that(response.status_code, equal_to(200))
    body = response.get_data(as_text=True)
    assert_that(body, contains_string(branch.name))
    for commit in commits:
        assert_that(body,
                    contains_string("/{0}/{1}/commits/{2}/".format(
                                    owner.name,
                                    repository.name,
                                    commit.sha)))
        assert_that(body,
                    contains_string("/commits/{0}".format(commit.sha)))
        assert_that(body,
                    contains_string("Everything is OK"))


def test_get_repository_doesnt_exist(app):
    """GET /{account}/{repository} raise 404 if not found."""

    tester = app.test_client()
    response = tester.get("/invenio/404/")

    assert_that(response.status_code, equal_to(404))



# -*- coding: utf-8 -*-
#
# This file is part of kwalitee
# Copyright (C) 2014, 2015 CERN.
#
# kwalitee is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# kwalitee is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with kwalitee; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#
# In applying this licence, CERN does not waive the privileges and immunities
# granted to it by virtue of its status as an Intergovernmental Organization
# or submit itself to any jurisdiction.

"""Kwalitee checks for PEP8, PEP257, PyFlakes and License."""

from __future__ import unicode_literals

import codecs
import os
import re
import tokenize

from datetime import datetime

import pep257

import pep8

import pyflakes
import pyflakes.checker


SUPPORTED_FILES = '.py', '.html', '.tpl', '.js', '.jsx', '.css', '.less'
"""Supported file types."""

_re_copyright_year = re.compile(r"^Copyright\s+(?:\([Cc]\)|\xa9)\s+"
                                r"(?:\d{4},\s+)*"
                                r"(?P<year>\d{4})\s+CERN\.?$",
                                re.UNICODE | re.MULTILINE)

_re_program = re.compile(r"^(?P<program>.*?) is free software;",
                         re.UNICODE | re.MULTILINE)
_re_program_2 = re.compile(r"^(?P<program>.*?) is distributed in",
                           re.UNICODE | re.MULTILINE)
_re_program_3 = re.compile(r"GNU General Public License\s+along\s+with "
                           r"(?P<program>.*?)[;\.]",
                           re.UNICODE | re.MULTILINE)

_re_bullet_label = re.compile(r"^\* (?P<label>[A-Z]{1,70}) ", re.UNICODE)

_messages_codes = {
    # Global
    "M100": "needs more reviewers",
    "M101": "signature is missing",
    "M102": "unrecognized bullet/signature",
    # First line
    "M110": "missing component name",
    "M111": "unrecognized component name: {0}",
    # Dots
    "M120": "missing empty line before bullet",
    "M121": "indentation of two spaces expected",
    "M122": "unrecognized bullet label: {0}",
    # Signatures
    "M130": "no bullets are allowed after signatures",
    # Generic
    "M190": "line is too long ({1} > {0})",
    "M191": "must not end with a dot '.'",
}

_licenses_codes = {
    "L100": "license is missing",
    "L101": "copyright is missing",
    "L102": "copyright year is outdated, expected {0} but got {1}",
    "L103": "license is not GNU GPLv2",
    "L190": "file cannot be decoded as {0}"
}


def _check_1st_line(line, **kwargs):
    """First line check.

    Check that the first line has a known component name followed by a colon
    and then a short description of the commit.

    :param line: first line
    :type line: str
    :param components: list of known component names
    :type line: list
    :param max_first_line: maximum length of the first line
    :type max_first_line: int
    :return: errors as in (code, line number, *args)
    :rtype: list

    """
    components = kwargs.get("components", ())
    max_first_line = kwargs.get("max_first_line", 50)

    errors = []
    lineno = 1
    if len(line) > max_first_line:
        errors.append(("M190", lineno, max_first_line, len(line)))

    if line.endswith("."):
        errors.append(("M191", lineno))

    if ':' not in line:
        errors.append(("M110", lineno))
    else:
        component, msg = line.split(':', 1)
        if component not in components:
            errors.append(("M111", lineno, component))

    return errors


def _check_bullets(lines, **kwargs):
    """Check that the bullet point list is well formatted.

    Each bullet point shall have one space before and after it. The bullet
    character is the "*" and there is no space before it but one after it
    meaning the next line are starting with two blanks spaces to respect the
    indentation.

    :param lines: all the lines of the message
    :type lines: list
    :param max_lengths: maximum length of any line. (Default 72)
    :return: errors as in (code, line number, *args)
    :rtype: list

    """
    max_length = kwargs.get("max_length", 72)
    labels = {l for l, _ in kwargs.get("commit_msg_labels", tuple())}

    errors = []
    missed_lines = []
    skipped = []

    for (i, line) in enumerate(lines[1:]):
        if line.startswith('*'):
            if len(missed_lines) > 0:
                errors.append(("M130", i + 2))
            if lines[i].strip() != '':
                errors.append(("M120", i + 2))

            label = _re_bullet_label.search(line)
            if label and label.group('label') not in labels:
                errors.append(("M122", i + 2, label.group('label')))

            for (j, indented) in enumerate(lines[i + 2:]):
                if indented.strip() == '':
                    break
                if not re.search(r"^ {2}\S", indented):
                    errors.append(("M121", i + j + 3))
                else:
                    skipped.append(i + j + 1)
        elif i not in skipped and line.strip():
            missed_lines.append((i + 2, line))

        if len(line) > max_length:
            errors.append(("M190", i + 2, max_length, len(line)))

    return errors, missed_lines


def _check_signatures(lines, **kwargs):
    """Check that the signatures are valid.

    There should be at least three signatures. If not, one of them should be a
    trusted developer/reviewer.

    Formatting supported being: [signature] full name <email@address>

    :param lines: lines (lineno, content) to verify.
    :type lines: list
    :param signatures: list of supported signature
    :type signatures: list
    :param alt_signatures: list of alternative signatures, not counted
    :type alt_signatures: list
    :param trusted: list of trusted reviewers, the e-mail address.
    :type trusted: list
    :param min_reviewers: minimal number of reviewers needed. (Default 3)
    :type min_reviewers: int
    :return: errors as in (code, line number, *args)
    :rtype: list

    """
    trusted = kwargs.get("trusted", ())
    signatures = tuple(kwargs.get("signatures", ()))
    alt_signatures = tuple(kwargs.get("alt_signatures", ()))
    min_reviewers = kwargs.get("min_reviewers", 3)

    matching = []
    errors = []
    signatures += alt_signatures

    test_signatures = re.compile("^({0})".format("|".join(signatures)))
    test_alt_signatures = re.compile("^({0})".format("|".join(alt_signatures)))
    for i, line in lines:
        if signatures and test_signatures.search(line):
            if line.endswith("."):
                errors.append(("M191", i))
            if not alt_signatures or not test_alt_signatures.search(line):
                matching.append(line)
        else:
            errors.append(("M102", i))

    if not matching:
        errors.append(("M101", 1))
        errors.append(("M100", 1))
    elif len(matching) < min_reviewers:
        pattern = re.compile('|'.join(map(lambda x: '<' + re.escape(x) + '>',
                                          trusted)))
        trusted_matching = list(filter(None, map(pattern.search, matching)))
        if len(trusted_matching) == 0:
            errors.append(("M100", 1))

    return errors


def check_message(message, **kwargs):
    """Check the message format.

    Rules:

    - the first line must start by a component name
    - and a short description (52 chars),
    - then bullet points are expected
    - and finally signatures.

    :param components: compontents, e.g. ``('auth', 'utils', 'misc')``
    :type components: `list`
    :param signatures: signatures, e.g. ``('Signed-off-by', 'Reviewed-by')``
    :type signatures: `list`
    :param alt_signatures: alternative signatures, e.g. ``('Tested-by',)``
    :type alt_signatures: `list`
    :param trusted: optional list of reviewers, e.g. ``('john.doe@foo.org',)``
    :type trusted: `list`
    :param max_length: optional maximum line length (by default: 72)
    :type max_length: int
    :param max_first_line: optional maximum first line length (by default: 50)
    :type max_first_line: int
    :param allow_empty: optional way to allow empty message (by default: False)
    :type allow_empty: bool
    :return: errors sorted by line number
    :rtype: `list`
    """
    if kwargs.pop("allow_empty", False):
        if not message or message.isspace():
            return []

    lines = re.split(r"\r\n|\r|\n", message)
    errors = _check_1st_line(lines[0], **kwargs)
    err, signature_lines = _check_bullets(lines, **kwargs)
    errors += err
    errors += _check_signatures(signature_lines, **kwargs)

    def _format(code, lineno, args):
        return "{0}: {1} {2}".format(lineno,
                                     code,
                                     _messages_codes[code].format(*args))

    return list(map(lambda x: _format(x[0], x[1], x[2:]),
                    sorted(errors, key=lambda x: x[0])))


class _PyFlakesChecker(pyflakes.checker.Checker):

    """PEP8 compatible checker for pyFlakes (inspired by flake8)."""

    name = "pyflakes"
    version = pyflakes.__version__

    def run(self):
        """Yield the error messages."""
        for msg in self.messages:
            col = getattr(msg, 'col', 0)
            yield msg.lineno, col, (msg.tpl % msg.message_args), msg.__class__


def _register_pyflakes_check():
    """Register the pyFlakes checker into PEP8 set of checks."""
    from flake8_import_order.flake8_linter import Linter
    from flake8_blind_except import check_blind_except

    # Resolving conflicts between pep8 and pyflakes.
    codes = {
        "UnusedImport": "F401",
        "ImportShadowedByLoopVar": "F402",
        "ImportStarUsed": "F403",
        "LateFutureImport": "F404",
        "Redefined": "F801",
        "RedefinedInListComp": "F812",
        "UndefinedName": "F821",
        "UndefinedExport": "F822",
        "UndefinedLocal": "F823",
        "DuplicateArgument": "F831",
        "UnusedVariable": "F841",
    }

    for name, obj in vars(pyflakes.messages).items():
        if name[0].isupper() and obj.message:
            obj.tpl = "{0} {1}".format(codes.get(name, "F999"), obj.message)

    pep8.register_check(_PyFlakesChecker, codes=['F'])
    # FIXME parser hack
    parser = pep8.get_parser('', '')
    Linter.add_options(parser)
    options, args = parser.parse_args([])
    Linter.parse_options(options)
    # end of hack
    pep8.register_check(Linter, codes=['I'])
    pep8.register_check(check_blind_except, codes=['B90'])
_registered_pyflakes_check = False


class _Report(pep8.BaseReport):

    """Custom reporter.

    It keeps a list of errors in a sortable list and never prints.
    """

    def __init__(self, options):
        """Initialize the reporter."""
        super(_Report, self).__init__(options)
        self.errors = []

    def error(self, line_number, offset, text, check):
        """Run the checks and collect the errors."""
        code = super(_Report, self).error(line_number, offset, text, check)
        if code:
            self.errors.append((line_number, offset + 1, code, text, check))


def is_file_excluded(filename, excludes):
    """Check if the file should be excluded.

    :param filename: file name
    :param excludes: list of regex to match
    :return: True if the file should be excluded
    """
    # check if you need to exclude this file
    return any([exclude and re.match(exclude, filename) is not None
                for exclude in excludes])


def check_pep8(filename, **kwargs):
    """Perform static analysis on the given file.

    :param filename: path of file to check.
    :type filename: str
    :param ignore: codes to ignore, e.g. ``('E111', 'E123')``
    :type ignore: `list`
    :param select: codes to explicitly select.
    :type select: `list`
    :param pyflakes: run the pyflakes checks too (default ``True``)
    :type pyflakes: bool
    :return: errors
    :rtype: `list`

    .. seealso:: :py:class:`pep8.Checker`

    """
    options = {
        "ignore": kwargs.get("ignore"),
        "select": kwargs.get("select"),
    }

    if not _registered_pyflakes_check and kwargs.get("pyflakes", True):
        _register_pyflakes_check()

    checker = pep8.Checker(filename, reporter=_Report, **options)
    checker.check_all()

    errors = []
    for error in sorted(checker.report.errors, key=lambda x: x[0]):
        errors.append("{0}:{1}: {3}".format(*error))
    return errors


def check_pep257(filename, **kwargs):
    """Perform static analysis on the given file docstrings.

    :param filename: path of file to check.
    :type filename: str
    :param ignore: codes to ignore, e.g. ('D400',)
    :type ignore: `list`
    :param match: regex the filename has to match to be checked
    :type match: str
    :param match_dir: regex everydir in path should match to be checked
    :type match_dir: str
    :return: errors
    :rtype: `list`

    .. seealso:: `GreenSteam/pep257 <https://github.com/GreenSteam/pep257/>`_

    """
    ignore = kwargs.get("ignore")
    match = kwargs.get("match", None)
    match_dir = kwargs.get("match_dir", None)

    errors = []

    if match and not re.match(match, os.path.basename(filename)):
        return errors

    if match_dir:
        # FIXME here the full path is checked, be sure, if match_dir doesn't
        # match the path (usually temporary) before the actual application path
        # it may not run the checks when it should have.
        path = os.path.split(os.path.abspath(filename))[0]
        while path != "/":
            path, dirname = os.path.split(path)
            if not re.match(match_dir, dirname):
                return errors

    checker = pep257.PEP257Checker()
    with open(filename) as fp:
        try:
            for error in checker.check_source(fp.read(), filename):
                if ignore is None or error.code not in ignore:
                    # Removing the colon ':' after the error code
                    message = re.sub("(D[0-9]{3}): ?(.*)",
                                     r"\1 \2",
                                     error.message)
                    errors.append("{0}: {1}".format(error.line, message))
        except tokenize.TokenError as e:
            errors.append("{1}:{2} {0}".format(e.args[0], *e.args[1]))
        except pep257.AllError as e:
            errors.append(str(e))

    return errors


def check_license(filename, **kwargs):
    """Perform a license check on the given file.

    The license format should be commented using # and live at the top of the
    file. Also, the year should be the current one.

    :param filename: path of file to check.
    :type filename: str
    :param year: default current year
    :type year: int
    :param ignore: codes to ignore, e.g. ``('L100', 'L101')``
    :type ignore: `list`
    :param python_style: False for JavaScript or CSS files
    :type python_style: bool
    :return: errors
    :rtype: `list`

    """
    year = kwargs.pop("year", datetime.now().year)
    python_style = kwargs.pop("python_style", True)
    ignores = kwargs.get("ignore")
    template = "{0}: {1} {2}"

    if python_style:
        re_comment = re.compile(r"^#.*|\{#.*|[\r\n]+$")
        starter = "# "
    else:
        re_comment = re.compile(r"^/\*.*| \*.*|[\r\n]+$")
        starter = " *"

    errors = []
    lines = []
    file_is_empty = False
    license = ""
    lineno = 0
    try:
        with codecs.open(filename, "r", "utf-8") as fp:
            line = fp.readline()
            blocks = []
            while re_comment.match(line):
                if line.startswith(starter):
                    line = line[len(starter):].lstrip()
                    blocks.append(line)
                    lines.append((lineno, line.strip()))
                lineno, line = lineno + 1, fp.readline()
            file_is_empty = line == ""
            license = "".join(blocks)
    except UnicodeDecodeError:
        errors.append((lineno + 1, "L190", "utf-8"))
        license = ""

    if file_is_empty and not license.strip():
        return errors

    match_year = _re_copyright_year.search(license)
    if match_year is None:
        errors.append((lineno + 1, "L101"))
    elif int(match_year.group("year")) != year:
        theline = match_year.group(0)
        lno = lineno
        for no, l in lines:
            if theline.strip() == l:
                lno = no
                break
        errors.append((lno + 1, "L102", year, match_year.group("year")))
    else:
        program_match = _re_program.search(license)
        program_2_match = _re_program_2.search(license)
        program_3_match = _re_program_3.search(license)
        if program_match is None:
            errors.append((lineno, "L100"))
        elif (program_2_match is None or
              program_3_match is None or
              (program_match.group("program").upper() !=
               program_2_match.group("program").upper() !=
               program_3_match.group("program").upper())):
            errors.append((lineno, "L103"))

    def _format_error(lineno, code, *args):
        return template.format(lineno, code,
                               _licenses_codes[code].format(*args))

    def _filter_codes(error):
        if not ignores or error[1] not in ignores:
            return error

    return list(map(lambda x: _format_error(*x),
                    filter(_filter_codes, errors)))


def check_file(filename, **kwargs):
    """Perform static analysis on the given file.

    .. seealso::

        - :data:`.SUPPORTED_FILES`
        - :func:`.check_pep8`
        - :func:`.check_pep257`
        - and :func:`.check_license`

    :param filename: path of file to check.
    :type filename: str
    :return: errors sorted by line number or None if file is excluded
    :rtype: `list`

    """
    excludes = kwargs.get("excludes", [])
    errors = []

    if is_file_excluded(filename, excludes):
        return None

    if filename.endswith(".py"):
        if kwargs.get("pep8", True):
            errors += check_pep8(filename, **kwargs)
        if kwargs.get("pep257", True):
            errors += check_pep257(filename, **kwargs)
        if kwargs.get("license", True):
            errors += check_license(filename, **kwargs)
    elif re.search("\.(tpl|html)$", filename):
        errors += check_license(filename, **kwargs)
    elif re.search("\.(js|jsx|css|less)$", filename):
        errors += check_license(filename, python_style=False, **kwargs)

    def try_to_int(value):
        try:
            return int(value.split(':', 1)[0])
        except ValueError:
            return 0

    return sorted(errors, key=try_to_int)


def get_options(config):
    """Build the options from the Flask config."""
    base = {
        "components": config.get("COMPONENTS"),
        "signatures": config.get("SIGNATURES"),
        "commit_msg_template": config.get("COMMIT_MSG_TEMPLATE"),
        "commit_msg_labels": config.get("COMMIT_MSG_LABELS"),
        "alt_signatures": config.get("ALT_SIGNATURES"),
        "trusted": config.get("TRUSTED_DEVELOPERS"),
        "pep8": config.get("CHECK_PEP8", True),
        "pep257": config.get("CHECK_PEP257", True),
        "license": config.get("CHECK_LICENSE", True),
        "pyflakes": config.get("CHECK_PYFLAKES", True),
        "ignore": config.get("IGNORE"),
        "select": config.get("SELECT"),
        "match": config.get("PEP257_MATCH"),
        "match_dir": config.get("PEP257_MATCH_DIR"),
        "min_reviewers": config.get("MIN_REVIEWERS"),
        "colors": config.get("COLORS", True),
        "excludes": config.get("EXCLUDES", [])
    }
    options = {}
    for k, v in base.items():
        if v is not None:
            options[k] = v
    return options
